@@ -1,5 +1,5 @@
 package swat;
-our $VERSION = v0.1.12;
+our $VERSION = v0.1.14;
 1;
 
 package main;
@@ -95,15 +95,28 @@ sub header {
 
 sub generate_asserts {
 
-    header();
 
-    my $patterns_file = shift;
+    my $filepath_or_array_ref = shift;
+    my $write_header = shift;
+
+    header() if $write_header;
+
+    my @ents = ();
+
+    if ( ref($filepath_or_array_ref) eq 'ARRAY') {
+        @ents = @$filepath_or_array_ref
+    }else{
+        open my $fh, $filepath_or_array_ref or die $!;
+        while (my $l = <$fh>){
+            push @ents, $l
+        }
+        close $fh;
+    }
 
-    open my $fh, $patterns_file or die $!;
 
     my $comment;
 
-    while (my $l = <$fh>){
+    for my $l (@ents){
 
         chomp $l;
         
@@ -115,11 +128,17 @@ sub generate_asserts {
             next;
         }
 
-        if ($l=~/\s*code:\s+(.*)/){
+        if ($l=~/^\s*code:\s+(.*)/){
             undef $comment;
             my $code = $1;
-            eval $code;            
-        }elsif($l=~/\s*regexp:\s+(.*)/){
+            eval $code;
+        }elsif($l=~/^\s*generator:\s+(.*)/){
+            undef $comment;
+            my $code = $1;
+            my $arr_ref = eval $code;
+            generate_asserts($arr_ref,0);
+            
+        }elsif($l=~/^\s*regexp:\s+(.*)/){
             my $re=$1;
             my $message = $comment ? "[$comment] $http_meth $path returns data matching $re" : "$http_meth $path returns data matching $re";
             check_line($re, 'regexp', $message);
@@ -132,7 +151,6 @@ sub generate_asserts {
         }
     }
 
-    close $fh;
 
 }
 
@@ -163,36 +181,42 @@ SWAT is Simple Web Application Test ( Tool )
 =head1 WHY
 
 I know there are a lot of tests tool and frameworks, but let me  briefly tell I<why> I created swat.
-As devops I update a dozens of web application weekly, sometimes I just have I<no time> sitting and wait while dev guys or QA team ensure that deploy is fine and nothing breaks on the road. So I need a B<tool to run smoke tests against web applications>. Not tool only, but the way to B<create such a tests from the scratch in way easy and fast enough>. So this how I came up with the idea of swat. If I was a marketing guy I'd say that swat:
+As devops I update a dozens of web application weekly, sometimes I just have I<no time> sitting and wait 
+while dev guys or QA team ensure that deploy is fine and nothing breaks on the road. 
+So I need a B<tool to run smoke tests against web applications>. 
+Not tool only, but the way to B<create such a tests from the scratch in way easy and fast enough>. 
+
+So this how I came up with the idea of swat. If I was a marketing guy I'd say that swat:
 
 =over
 
-=item -
+=item *
 
 is easy to use and flexible tool to run smoke tests against web applications
 
 
-=item -
+=item *
 
 is L<curl|http://curl.haxx.se/> powered and L<TAP|https://testanything.org/> compatible
 
+=item *
 
-=item -
+leverages famous L<prove|http://search.cpan.org/perldoc?prove> utility
+
+
+=item *
 
 has minimal dependency tree  and probably will run out of the box on most linux environments, provided that one has perl/bash/find/curl by hand ( which is true  for most cases )
 
-
-=item -
+=item *
 
 has a simple and yet powerful DSL allow you to both run simple tests ( 200 OK ) or complicated ones ( using curl api and perl one-liners calls )
 
-
-=item -
+=item *
 
 is daily it/devops/dev helper with low price mastering ( see my tutorial )
 
-
-=item -
+=item *
 
 and yes ... swat is fun :)
 
@@ -206,16 +230,15 @@ and yes ... swat is fun :)
 =head2 Install swat
 
 
-=head2 developer release
+=head3 developer release
 
     sudo cpanm --mirror-only --mirror https://stratopan.com/melezhik/swat-release/master swat
 
-
-=head2 stable release
+=head3 stable release
 
     sudo cpan install swat
 
-Once swat is installed you have swat command line tool to run swat tests, but before do this you need to create them.
+Once swat is installed you have B<swat> command line tool to run swat tests, but before do this you need to create them.
 
 
 =head2 Create tests
@@ -240,17 +263,15 @@ Once swat is installed you have swat command line tool to run swat tests, but be
 
     swat ./my-app http://127.0.0.1
 
-
 =head1 DSL
 
 Swat DSL consists of 2 parts. Routes and check patterns.
 
-
 =head2 Routes
 
 Routes are http resources a tested web application should has.
 
-Swat utilize file system data calculating all existed routes as sub directories paths in the project root directory.
+Swat utilize file system I<representing> all existed routes as sub directories paths in the project root directory.
 Let we have a following project layout:
 
     example/my-app/
@@ -262,7 +283,7 @@ When you give swat a run
 
     swat example/my-app 127.0.0.1
 
-It just find all the directories holding get.txt files and "create" routes:
+It will find all the directories holding get.txt files and "create" routes:
 
     GET hello/
     GET hello/world
@@ -276,17 +297,17 @@ As you can see from tutorial above check patterns are  just text files describin
 
 =over
 
-=item -
+=item *
 
 Expected Values
 
 
-=item -
+=item *
 
 Comments
 
 
-=item -
+=item *
 
 Perl one-liners code
 
@@ -296,7 +317,7 @@ Perl one-liners code
 
 =head3 Expected values
 
-This is most usable entity that one may define at check patterns files. I<It's just s string should be returned> when swat request a given URI. Here are examples:
+This is most usable entity that one may define at check patterns files. I<It's just a string should be returned> when swat request a given URI. Here are examples:
 
     200 OK
     Hello World
@@ -305,7 +326,7 @@ This is most usable entity that one may define at check patterns files. I<It's j
 
 =head3 Comments
 
-Comments are lines started with '#' symbol, they are for human not for swat which ignore them when parse check pattern file. Here are examples.
+Comments are lines started with '#' symbol, they are for humans not for swat which ignore comments when parse check pattern file. Here are examples.
 
     # this http status is expected
     200 OK
@@ -316,7 +337,7 @@ Comments are lines started with '#' symbol, they are for human not for swat whic
 =head3 Perl one-liners code
 
 Everything started with C<code:> would be treated by swat as perl code to execute.
-There are a I<lot of> possibilities! Please follow L<Test::More|search.cpan.org/perldoc/Test::More> documentation to get more info about useful function you may call here.
+There are a I<lot of possibilities>! Please follow L<Test::More|search.cpan.org/perldoc/Test::More> documentation to get more info about useful function you may call here.
 
     code: skip('next test is skipped',1) # skip next check forever
     HELLO WORLD
@@ -338,37 +359,30 @@ When talking about swat I always say about Get http request, but swat may send a
     echo 200 OK >> my-app/hello/post.txt
     echo 200 OK >> my-app/hello/world/post.txt
 
-You may use curl_params settings ( follow swat settings section for details ) to define post data, there are examples:
+You may use curl_params setting ( follow L</"Swat settings"> section for details ) to define post data, there are some examples:
 
 =over
 
-=item -
+=item *
 
 C<-d> - Post data sending by html form submit.
 
 
-=back
-
-
      # Place this in swat.ini file or sets as env variable:
      curl_params='-d name=daniel -d skill=lousy'
 
 
-=over
-
-=item -
+=item *
 
 C<--data-binary> - Post data sending as is.
 
 
-=back
-
-
      # Place this in swat.ini file or sets as env variable:
      curl_params=`echo -E "--data-binary '{\"name\":\"alex\",\"last_name\":\"melezhik\"}'"`
      curl_params="${curl_params} -H 'Content-Type: application/json'"
 
 
+=back
 
 =head1 Swat settings
 
@@ -376,12 +390,12 @@ Swat comes with settings defined in two contexts:
 
 =over
 
-=item -
+=item *
 
 environmental variables
 
 
-=item -
+=item *
 
 swat.ini files
 
@@ -395,44 +409,49 @@ Defining a proper environment variables will provide swat settings.
 
 =over
 
-=item -
+=item *
 
-debug - set to 1 if you want to see some debug information in output, default value is C<0>
+C<debug> - set to C<1> if you want to see some debug information in output, default value is C<0>
 
 
-=item -
+=item *
 
-debug_bytes - number of bytes of http response  to be dumped out when debug is on. default value is C<500>
+C<debug_bytes> - number of bytes of http response  to be dumped out when debug is on. default value is C<500>
 
 
-=item -
+=item *
 
-ignoreI<http>err - ignore http errors, if this parameters is off (set to C<1>) returned  I<error http codes> will not result in test fails, useful when one need to test something with response differ from  2**,3** http codes. Default value is C<0>
+C<ignore_http_err> - ignore http errors, if this parameters is off (set to C<1>) returned  I<error http codes> will not result in test fails, 
+useful when one need to test something with response differ from  2**,3** http codes. Default value is C<0>
 
 
-=item -
+=item *
 
-try_num - number of http requests  attempts before give it up ( useless for resources with slow response  ), default value is C<2>
+C<try_num> - number of http requests  attempts before give it up ( useless for resources with slow response  ), default value is C<2>
 
 
-=item -
+=item *
 
-curl_params - additional curl parameters being add to http requests, default value is C<"">, follow curl documentation for variety of values for this
+C<curl_params> - additional curl parameters being add to http requests, default value is C<"">, follow curl documentation for variety of values for this
 
 
-=item -
+=item *
 
-curlI<connect>timeout - follow curl documentation
+C<curl_connec_timeout> - follow curl documentation
 
 
-=item -
+=item *
 
-curlI<max>time - follow curl documentation
+C<curl_max_time> - follow curl documentation
 
 
-=item -
+=item *
 
-port  - http port of tested host, default value is '80'
+C<port>  - http port of tested host, default value is C<80>
+
+=item *
+
+C<noproxy>  - ignore http proxy when making http requests, default value is C<1>
 
 
 =back
@@ -441,9 +460,22 @@ port  - http port of tested host, default value is '80'
 =head2 Swat.ini files
 
 Swat checks files named C<swat.ini> in the following directories
-- ~/swat.ini
-- $project_root_directory/swat.ini
-- $route_directory/swat.ini
+
+=over
+
+=item *
+
+B<~/swat.ini>
+
+=item *
+
+B<$project_root_directory/swat.ini>
+
+=item *
+
+B<$route_directory/swat.ini>
+
+=back
 
 Here are examples of locations of swat.ini files:
 
@@ -456,7 +488,7 @@ Here are examples of locations of swat.ini files:
      my-app/hello/world/swat.ini # route directory swat.ini file ( route hello/world )
 
 
-Once file exists at ay location swat simply B<bash source it> to apply settings
+Once file exists at any location swat simply B<bash sources it> to apply settings.
 
 Thus swat.ini file should be bash file with swat variables definitions. Here is example:
 
@@ -469,61 +501,85 @@ Thus swat.ini file should be bash file with swat variables definitions. Here is
 
 Here is the list of settings/contexts  in priority ascending order:
 
-| context | location | priority  level |
-| --------| ----- | --------- |---- |
-| swat.ini file           | ~/swat.ini              | 1 |
-| environmental variables | ---                     | 2 |
-| swat.ini file           | project root directory  | 3 |
-| swat.ini file           | route directory         | 4 |
+    | context                 | location                | priority  level |
+    | ------------------------|------------------------ | --------------- |
+    | swat.ini file           | ~/swat.ini              |               1 |
+    | environmental variables | ---                     |               2 |
+    | swat.ini file           | project root directory  |               3 |
+    | swat.ini file           | route directory         |               4 |
 
-Swat process settings in order. For every route found swat:
-- Clear all settings
-- Apply settings from environmental variables ( if any given )
-- Apply settings from swat.ini file in home directory ( if any given )
-- Apply settings from swat.ini file in project root directory ( if any given )
-- And finally apply settings from swat.ini file in route directory ( if any given )
+
+Swat processes settings I<in order>. For every route found swat:
+
+=over
+
+=item *
+
+Clear all settings
+
+=item *
+
+Apply settings from environmental variables ( if any given )
+
+=item *
+
+Apply settings from swat.ini file in home directory ( if any given )
+
+
+=item *
+
+Apply settings from swat.ini file in project root directory ( if any given )
+
+=item *
+
+And finally apply settings from swat.ini file in route directory ( if any given )
+
+=back
 
 
 =head1 TAP
 
 Swat produce output in L<TAP|https://testanything.org/> format , that means you may use your favorite tap parsers to bring result to
-another test / reporting systems, follow TAP documentation to get more on this.
+another test / reporting systems, follow TAP documentation to get more on this. Here is example for converting swat tests into JUNIT format
 
+    swat $project_root $host --formatter TAP::Formatter::JUnit
+
+
+See also L<"Prove settings"> section.
 
 =head1 Command line tool
 
-Swat is shipped as cpan package , once it's installed ( see install section ) you have a command line tool called C<swat>, this is usage info on it:
+Swat is shipped as cpan package, once it's installed ( see L</"Install swat"> section ) you have a command line tool called B<swat>, this is usage info on it:
 
-    swat project_dir URL <options>
+    swat project_dir URL <prove settings>
 
 =over
 
-=item -
+=item *
 
-URL - is base url for web application you run tests against, you need defined routes which will be requested against URL, see DSL section.
+B<URL> - is base url for web application you run tests against, you need defined routes which will be requested against URL, see DSL section.
 
 
-=item -
+=item *
 
-project_dir - is a project root directory
+B<project_dir> - is a project root directory
 
 
 =back
 
 
-=head2 options
+=head2 Prove settings
 
-As swat I<uses prove utility> to run tests, all the swat options are passed as is to prove utility.
+Swat utilize L<prove utility|http://search.cpan.org/perldoc?prove> to run tests, so all the swat options I<are passed as is to prove utility>.
 Follow L<prove|http://search.cpan.org/perldoc?prove> utility documentation for variety of values you may set here.
-Default value for options is  C<-v>. Here is another examples:
+Default value for prove options is  C<-v>. Here is another examples:
 
 =over
 
-=item -
+=item *
 
 C<-q -s> -  run tests in random and quite mode
 
-
 =back
 
 
@@ -538,14 +594,28 @@ Not that many :)
 
 =over
 
-=item -
+=item *
 
-perl / curl / bash / find  / head
+perl 
 
+=item *
+
+curl 
+
+=item *
+
+bash
+
+=item *
+
+find
+
+=item *
+
+head
 
 =back
 
-
 =head1 AUTHOR
 
 L<Aleksei Melezhik|mailto:melezhik@gmail.com>
@@ -554,9 +624,27 @@ L<Aleksei Melezhik|mailto:melezhik@gmail.com>
 =head1 Thanks
 
 To the authors of ( see list ) without who swat would not appear to light
-- perl
-- curl
-- TAP
-- Test::More
-- prove
 
+=over
+
+=item *
+
+perl
+
+=item *
+
+curl
+
+=item *
+
+TAP
+
+=item *
+
+Test::More
+
+=item *
+
+prove
+
+=back
